/*
    lkpatch - a Linux Kernel Patcher

    Copyright 1999, 2000 Theo Nolte <nolte@post.rwth-aachen.de>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <stdarg.h>
#include <unistd.h>

#include "lkpatch.h"

void error(int exstat,int enr,char* msg,...) {
 va_list ap;
  va_start(ap,msg);
  vfprintf(stderr,msg,ap);
  va_end(ap);
  if(enr) fprintf(stderr," (%s)",strerror(enr));
  fprintf(stderr,"\n");
  if(exstat) exit(exstat);
}

static int debug=1;

int dlvl(void) { return(debug); }
void setdlvl(int i) { debug=i; }

void dp(int i,char*fmt,...) {
 va_list ap;
 int nl=1;
 char*s;
  if(i>debug) return;
  va_start(ap,fmt);
  s=estrdup(fmt);
  if(s[strlen(s)-1]=='-') nl=s[strlen(s)-1]=0;
/*  fprintf(stderr,"");*/
  vfprintf(stderr,s,ap);
  if(nl) fprintf(stderr,"\n");
  va_end(ap);
  free(s);
}

int stosec(char*s) { /* convert s to nr of seconds */
 int i,n; char*start,*end;
  dp(DTRC,"stosec(%s)-",s);
  for(start=s,n=0;*start;start=end+1,n+=i) {
    i=strtol(start,&end,10);
    switch(*end) {
      case '\0': end--; break;
      case 's': break;
      case 'm': i*=60; break;
      case 'h': i*=3600; break;
      case 'd': i*=86400; break;
      default: error(1,0,"bad time spec %s",s);
    }
  }
  dp(DTRC," : %d",n);
  return(n);
}

/*------------------------------------------------------------------*/

void*emalloc(int sz) {
 void*rv=malloc(sz);
  if(!rv) error(1,errno,"emalloc: out of mem");
  return(rv);
}

void*estrdup(char*s) {
 char*rv=strdup(s);
  if(!rv) error(1,errno,"estrdup: out of mem");
  return(rv);
}

char*progpath(char*prog) {
 char*t,*path;
  dp(DALL,"progpath(%s)",prog);
  if(getenv("PATH")) path=estrdup(getenv("PATH"));
  else path=estrdup("/bin:/usr/bin:/sbin:/usr/sbin");
  while((t=strsep(&path,":"))) {
   char*s;
/*printf("token: %s\n",t);*/
    s=emalloc(strlen(t)+1+strlen(prog)+1);
    sprintf(s,"%s%s%s",t,s[strlen(s)-1]!='/'?"/":"",prog);
/*printf("trying prog %s\n",s);*/
    if(!access(s,X_OK)) {
      free(path);
      dp(DTRC,"progpath(%s): rtn %s",prog,s);
      return(s);
    }
    free(s);
  }
  return(0);
}
